'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _angular = require('angular');

var _proptypes = require('proptypes');

var _proptypes2 = _interopRequireDefault(_proptypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function check(props, propTypes, name) {
  for (var prop in propTypes) {
    if (propTypes.hasOwnProperty(prop)) {
      var err = propTypes[prop](props, prop, name, 'prop');
      if (err) {
        console.warn(err);
        return false;
      }
    }
  }
  return true;
}

function makeDirectiveApiSuggested(aDirectiveRegisterFun) {
  return function (name, directiveFactory) {
    var directiveFun = (0, _angular.isFunction)(directiveFactory) ? directiveFactory : directiveFactory.slice(-1)[0];
    var finalDirectiveFactory = [].concat(directiveFactory);

    function wrappedDirFun() {
      var dirDef = directiveFun.apply(undefined, arguments);

      if (!dirDef._propTypes_) {
        console.warn('directive **' + name + '** not having _propTypes_ defined.\nDefining the _propTypes_ allow developer easy to understand which data should provide to to directive');
      }

      var dirLinkFun = dirDef.link;

      function wrappedDirLinkFun($scope) {
        if (check($scope, dirDef._propTypes_, name)) {
          for (var _len = arguments.length, more = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
            more[_key - 1] = arguments[_key];
          }

          return dirLinkFun.apply(undefined, [$scope].concat(more));
        }
        throw new Error('properties of directive **' + name + '** is not provided correctly!');
      }
      dirDef.link = wrappedDirLinkFun;

      return dirDef;
    }

    finalDirectiveFactory.splice(-1, 1, wrappedDirFun);

    return aDirectiveRegisterFun(name, finalDirectiveFactory);
  };
}

exports.default = makeDirectiveApiSuggested;